/*----------------------------------------------------------------------*\
 | dbfphp.c is a program that reads a DBF file and generates from it	|
 | a set of PHP statements that create an array of arrays for use in	|
 | the function dbase_create().											|
 |																		|
 | Based on the dbf access routines developed by Frank Warmerdam as		|
 | part of his shapelib C library for reading and writing shapefiles.	|
 | <http://shapelib.maptools.org/>  Thanks, Frank!						|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <unistd.h>

#include "shapefil.h"
#include "stricmp.h"

extern int is_reserved (char *name);

static char version[] = "1.0";

/*----------------------------------------------------------------------*\
 | Revision history:													|
 |																		|
\*----------------------------------------------------------------------*/

struct field_info {
	char name [16];
	DBFFieldType type;
	int width;
	int precision;
	};

static char *type_of (DBFFieldType t) {
	if (t == FTString)  return ("varchar");
	if (t == FTInteger) return ("int");
	if (t == FTDouble)  return ("real");
	if (t == FTInvalid) return ("invalid");
	return ("unknown");
	}

static char *dbftype_of (DBFFieldType t) {
	if (t == FTString)  return ("C");
	if (t == FTInteger) return ("N");
	if (t == FTDouble)  return ("N");
	if (t == FTInvalid) return ("L");
	return ("unknown");
	}


#define MAX_TEXT_LENGTH	4096

int main (int argc, char *argv[]) {
	char *input_file = NULL;
	char output_file [FILENAME_MAX];
	char table_name [FILENAME_MAX];
	char text [MAX_TEXT_LENGTH];
	FILE *out;
	DBFHandle df;
	int rc,fc,max_width;
	int i,j,k;
	DBFFieldType ft[1024];
	char *s,*t;
	struct field_info *field = NULL;
	int do_inserts = 1;

	*output_file = 0;
	if (argc > 1)
		for (i=1; i < argc; i++)
			if (memcmp (argv[i],"-o",2) == 0) {
				i++;
				if (i < argc) strcpy (output_file,argv[i]);
				}
			else
				if (memcmp (argv[i],"-c",2) == 0) {
					do_inserts = 0;
					}
				else
					input_file = argv[i];

	printf ("dbfphp %s - Peter N. Schweitzer (U.S. Geological Survey)\n",version);

	if (input_file) {

		/* Open and process the input file */

		if (df = DBFOpen (input_file,"rb")) {

			rc = DBFGetRecordCount (df);
			fc = DBFGetFieldCount (df);

			printf ("Opened file %s\n",input_file);
			printf ("  Number of fields: %d\n",fc);
			printf ("  Number of records: %d\n",rc);

			/* Figure table name from input file */
			strcpy (table_name,input_file);
			if (s = strrchr (table_name,'.')) *s = 0;
			if (s = strrchr (table_name,'/')) strcpy (table_name,s+1);
			printf ("  Table name will be %s\n",table_name);

			/* Allocate space for the field info */

			if (field = (struct field_info *) malloc (fc * sizeof (struct field_info))) {
				for (i=0; i < fc; i++)
					field[i].type = DBFGetFieldInfo (df,i,field[i].name,&field[i].width, &field[i].precision);

				/*------------------------------------------------------*\
				 | Look at the names to make sure there aren't any		|
				 | duplicate names.  Warn user if there are.			|
				\*------------------------------------------------------*/

				for (i=0; i < fc - 1; i++)
					for (j=i+1; j < fc; j++)
						if (strcmp (field[i].name,field[j].name) == 0)
							printf ("WARNING: Fields %d and %d have the same name \"%s\"\n",i+1,j+1,field[i].name);
				}

			/*----------------------------------------------------------*\
			 | If the output file is not specified with -o, then make	|
			 | it the same as the input file with .php instead of .dbf.	|
			\*----------------------------------------------------------*/

			if (!*output_file) {
				strcpy (output_file,input_file);
				if (s = strrchr (output_file,'.')) *s = 0;
				strcat (output_file,".php");
				}
			printf ("  Output file name: %s\n",output_file);

			if (out = fopen (output_file,"w")) {
				/* Make the PHP array of arrays */

				fprintf (out,"array (\n");
				for (j=0; j < fc; j++) {

					/*--------------------------------------------------*\
					 | Gotta make sure the field name is not reserved	|
					 | by SQL (or in this case by MySQL).				|
					 |													|
					 | I do this by appending an underscore to the name.|
					 | This will always work because the names in the	|
					 | DBF file cannot be more than 10 characters, and	|
					 | I have allocated 16 for the name in the field	|
					 | structure here.  SQL allows longer names.		|
					\*--------------------------------------------------*/

					if (is_reserved (field[j].name))
						strcat (field[j].name,"_");

					fprintf (out,"\tarray (\"%s\",",field[j].name);

					fprintf (out,"\"%s\"",dbftype_of(field[j].type));
					switch (field[j].type) {
						case FTString:
							fprintf (out,",%d",field[j].width);
							break;
						case FTDouble:
						case FTInteger:
							fprintf (out,",%d",field[j].width);
							fprintf (out,",%d",field[j].precision);
							break;
						default:
							break;
						}
					fprintf (out,")");
					if (j < fc - 1) fprintf (out,",");
					fprintf (out,"\n");
					}
				fprintf (out,"\t);\n");

				fclose (out);
				}
			else {
				printf ("Error: could not create output file %s\n",output_file);
				DBFClose (df);
				exit (1);
				}
			DBFClose (df);
			}
		else {
			printf ("Error: could not open input file %s\n",input_file);
			exit (1);
			}
		}
	else {
		printf ("Usage: %s input_file [-o output_file]\n",argv[0]);
		exit (0);
		}
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
